@@ -11,6 +11,7 @@ from typing import TYPE_CHECKING, cast
 from aspy.refactor_imports.classify import ImportType, classify_import
 
 from flake8_type_checking.constants import (
+    ANNOTATION_PROPERTY,
     ATTRIBUTE_PROPERTY,
     ATTRS_DECORATORS,
     ATTRS_IMPORTS,
@@ -34,11 +35,6 @@ if TYPE_CHECKING:
 
     from flake8_type_checking.types import Flake8Generator, FunctionRangesDict, FunctionScopeImportsDict, Import, Name
 
-    MixinBase = 'ImportVisitor'
-
-else:
-    MixinBase = object
-
 
 class AttrsMixin:
     """
@@ -48,7 +44,8 @@ class AttrsMixin:
     we treat type hints on attrs classes as needed at runtime.
     """
 
-    third_party_imports: dict[str, Import]
+    if TYPE_CHECKING:
+        third_party_imports: dict[str, Import]
 
     def get_all_attrs_imports(self) -> dict[Optional[str], str]:
         """Return a map of all attrs/attr imports."""
@@ -98,7 +95,7 @@ class AttrsMixin:
         return actual in ATTRS_DECORATORS
 
 
-class DunderAllMixin(MixinBase):  # type: ignore
+class DunderAllMixin:
     """
     Contains the necessary logic for preventing __all__ false positives.
 
@@ -117,6 +114,12 @@ class DunderAllMixin(MixinBase):  # type: ignore
     this is an exception to the general rule.
     """
 
+    if TYPE_CHECKING:
+        uses: dict[str, ast.AST]
+
+        def generic_visit(self, node: ast.AST) -> None:  # noqa: D102
+            ...
+
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         self.__all___assignments: list[tuple[int, int]] = []
 
@@ -170,7 +173,46 @@ class DunderAllMixin(MixinBase):  # type: ignore
         return node
 
 
-class FastAPIMixin(MixinBase):  # type: ignore
+class PydanticMixin:
+    """
+    Contains the necessary logic for Pydantic support.
+
+    At least the code that could be separated from the main visitor class.
+    Some pydantic stuff is still contained in the main class.
+    """
+
+    if TYPE_CHECKING:
+        pydantic_enabled: bool
+        pydantic_validate_arguments_import_name: Optional[str]
+
+        def visit(self, node: ast.AST) -> ast.AST:  # noqa: D102
+            ...
+
+    def _function_is_wrapped_by_validate_arguments(self, node: Union[FunctionDef, AsyncFunctionDef]) -> bool:
+        if self.pydantic_enabled and node.decorator_list:
+            for decorator_node in node.decorator_list:
+                if getattr(decorator_node, 'id', '') == self.pydantic_validate_arguments_import_name:
+                    return True
+        return False
+
+    def visit_FunctionDef(self, node: FunctionDef) -> None:
+        """Remove and map function arguments and returns."""
+        if self._function_is_wrapped_by_validate_arguments(node):
+            for path in [node.args.args, node.args.kwonlyargs, node.args.posonlyargs]:
+                for argument in path:
+                    if hasattr(argument, 'annotation') and argument.annotation:
+                        self.visit(argument.annotation)
+
+    def visit_AsyncFunctionDef(self, node: AsyncFunctionDef) -> None:
+        """Remove and map function arguments and returns."""
+        if self._function_is_wrapped_by_validate_arguments(node):
+            for path in [node.args.args, node.args.kwonlyargs, node.args.posonlyargs]:
+                for argument in path:
+                    if hasattr(argument, 'annotation') and argument.annotation:
+                        self.visit(argument.annotation)
+
+
+class FastAPIMixin:
     """
     Contains the necessary logic for FastAPI support.
 
@@ -178,16 +220,22 @@ class FastAPIMixin(MixinBase):  # type: ignore
     to treat annotations as needed at runtime.
     """
 
-    fastapi_enabled: bool
-    fastapi_dependency_support_enabled: bool
+    if TYPE_CHECKING:
+        fastapi_enabled: bool
+        fastapi_dependency_support_enabled: bool
 
-    def visit_FunctionDef(self, node: ast.FunctionDef) -> None:
+        def visit(self, node: ast.AST) -> ast.AST:  # noqa: D102
+            ...
+
+    def visit_FunctionDef(self, node: FunctionDef) -> None:
         """Remove and map function arguments and returns."""
+        super().visit_FunctionDef(node)  # type: ignore[misc]
         if (self.fastapi_enabled and node.decorator_list) or self.fastapi_dependency_support_enabled:
             self.handle_fastapi_decorator(node)
 
-    def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef) -> None:
+    def visit_AsyncFunctionDef(self, node: AsyncFunctionDef) -> None:
         """Remove and map function arguments and returns."""
+        super().visit_AsyncFunctionDef(node)  # type: ignore[misc]
         if (self.fastapi_enabled and node.decorator_list) or self.fastapi_dependency_support_enabled:
             self.handle_fastapi_decorator(node)
 
@@ -248,7 +296,7 @@ class ImportName:
                      ^-- this
 
             from pandas import DataFrame
-                                 ^--this
+                                  ^--this
 
             from pandas import DataFrame as df
                                             ^-- or this
@@ -287,7 +335,7 @@ class ImportName:
         return cast(ImportTypeValue, classify_import(self.full_name))
 
 
-class ImportVisitor(DunderAllMixin, AttrsMixin, FastAPIMixin, ast.NodeVisitor):
+class ImportVisitor(DunderAllMixin, AttrsMixin, FastAPIMixin, PydanticMixin, ast.NodeVisitor):
     """Map all imports outside of type-checking blocks."""
 
     def __init__(
@@ -308,6 +356,7 @@ class ImportVisitor(DunderAllMixin, AttrsMixin, FastAPIMixin, ast.NodeVisitor):
         self.fastapi_dependency_support_enabled = fastapi_dependency_support_enabled
         self.cattrs_enabled = cattrs_enabled
         self.pydantic_enabled_baseclass_passlist = pydantic_enabled_baseclass_passlist
+        self.pydantic_validate_arguments_import_name = None
         self.cwd = cwd  # we need to know the current directory to guess at which imports are remote and which are not
 
         #: Import patterns we want to avoid mapping
@@ -446,7 +495,7 @@ class ImportVisitor(DunderAllMixin, AttrsMixin, FastAPIMixin, ast.NodeVisitor):
 
     # -- Map imports -------------------------------
 
-    def add_import(self, node: Import) -> None:
+    def add_import(self, node: Import) -> None:  # noqa: C901
         """Add relevant ast objects to import lists."""
         if self.in_type_checking_block(node):
             # For type checking blocks we want to
@@ -474,6 +523,13 @@ class ImportVisitor(DunderAllMixin, AttrsMixin, FastAPIMixin, ast.NodeVisitor):
             if name_node.name == 'TYPE_CHECKING' and name_node.asname is not None:
                 self.type_checking_alias = name_node.asname
 
+            # Look for pydantic.validate_arguments import
+            if name_node.name == 'validate_arguments':
+                if name_node.asname is not None:
+                    self.pydantic_validate_arguments_import_name = name_node.asname
+                else:
+                    self.pydantic_validate_arguments_import_name = name_node.name
+
             # Look for typing import
             if name_node.name == 'typing' and name_node.asname is not None:
                 self.typing_alias = name_node.asname
@@ -561,11 +617,27 @@ class ImportVisitor(DunderAllMixin, AttrsMixin, FastAPIMixin, ast.NodeVisitor):
 
     def visit_Name(self, node: ast.Name) -> ast.Name:
         """Map names."""
+        if hasattr(node, ANNOTATION_PROPERTY):
+            # Skip handling of annotation objects
+            return node
+
         if self.in_type_checking_block(node):
             return node
 
         if hasattr(node, ATTRIBUTE_PROPERTY):
-            self.uses[f'{node.id}.{getattr(node, ATTRIBUTE_PROPERTY)}'] = node
+            # When importing modules like 'import urllib.parse'
+            # and calling 'urllib.parse.urlencode(some_string)',
+            # we only receive 'urllib' here as the node.id, but can
+            # construct 'urllib.parse.urlencode' by using the attribute property.
+            # Alas, the import in this case would only match 'urllib.parse', so for
+            # this case, we add two uses in these cases where the link-length is gt 2
+            full_name = f'{node.id}.{getattr(node, ATTRIBUTE_PROPERTY)}'
+            split_names = full_name.split('.')
+            if len(split_names) > 2:
+                self.uses[full_name] = node
+                self.uses['.'.join(split_names[:-1])] = node
+            else:
+                self.uses[full_name] = node
 
         self.uses[node.id] = node
         return node
@@ -577,33 +649,24 @@ class ImportVisitor(DunderAllMixin, AttrsMixin, FastAPIMixin, ast.NodeVisitor):
 
     def add_annotation(self, node: ast.AST) -> None:
         """Map all annotations on an AST node."""
-        if isinstance(node, ast.Ellipsis):
+        if isinstance(node, (ast.Ellipsis, ast.BinOp)) or node is None:
             return
-        if py38 and isinstance(node, Index):
-            return self.add_annotation(node.value)
-        if isinstance(node, ast.Constant):
-            if node.value is None:
-                return
-            self.wrapped_annotations.append((node.lineno, node.col_offset, node.value))
-        elif isinstance(node, ast.Subscript):
-            value = cast(ast.Name, node.value)
-            if hasattr(value, 'id') and value.id == 'Literal':
-                # Type hinting like `x: Literal['one', 'two', 'three']`
-                # creates false positives unless excluded
-                return
+        elif (py38 and isinstance(node, Index)) or isinstance(node, ast.Attribute):
+            self.add_annotation(node.value)
+        elif isinstance(node, ast.Subscript) and getattr(node.value, 'id', '') != 'Literal':
             self.add_annotation(node.value)
             self.add_annotation(node.slice)
-        elif isinstance(node, ast.Name):
-            self.unwrapped_annotations.append((node.lineno, node.col_offset, node.id))
         elif isinstance(node, (ast.Tuple, ast.List)):
             for n in node.elts:
                 self.add_annotation(n)
-        elif node is None:
-            return
-        elif isinstance(node, ast.Attribute):
-            self.add_annotation(node.value)
-        elif isinstance(node, ast.BinOp):
-            return
+        elif isinstance(node, ast.Constant) and node.value is not None:
+            # Register annotation value
+            setattr(node, ANNOTATION_PROPERTY, True)
+            self.wrapped_annotations.append((node.lineno, node.col_offset, node.value))
+        elif isinstance(node, ast.Name):
+            # Register annotation value
+            setattr(node, ANNOTATION_PROPERTY, True)
+            self.unwrapped_annotations.append((node.lineno, node.col_offset, node.id))
 
     @staticmethod
     def set_child_node_attribute(node: Any, attr: str, val: Any) -> Any:
@@ -671,43 +734,28 @@ class ImportVisitor(DunderAllMixin, AttrsMixin, FastAPIMixin, ast.NodeVisitor):
 
         And we also note down the start and end line number for the function.
         """
-        for path in [node.args.args, node.args.kwonlyargs]:
+        for path in [node.args.args, node.args.kwonlyargs, node.args.posonlyargs]:
             for argument in path:
                 if hasattr(argument, 'annotation') and argument.annotation:
                     self.add_annotation(argument.annotation)
-                    delattr(argument, 'annotation')
 
-        if (
-            hasattr(node.args, 'kwarg')
-            and node.args.kwarg
-            and hasattr(node.args.kwarg, 'annotation')
-            and node.args.kwarg.annotation
-        ):
-            self.add_annotation(node.args.kwarg.annotation)
-            delattr(node.args.kwarg, 'annotation')
-
-        if (
-            hasattr(node.args, 'vararg')
-            and node.args.vararg
-            and hasattr(node.args.vararg, 'annotation')
-            and node.args.vararg.annotation
-        ):
-            self.add_annotation(node.args.vararg.annotation)
-            delattr(node.args.vararg, 'annotation')
+        path_: str
+        for path_ in ['kwarg', 'vararg']:
+            if (arg := getattr(node.args, path_, None)) and getattr(arg, 'annotation', None):
+                self.add_annotation(arg.annotation)
 
         if hasattr(node, 'returns') and node.returns:
             self.add_annotation(node.returns)
-            delattr(node, 'returns')
 
         self.register_function_ranges(node)
 
-    def visit_FunctionDef(self, node: ast.FunctionDef) -> None:
+    def visit_FunctionDef(self, node: FunctionDef) -> None:
         """Remove and map function argument- and return annotations."""
         super().visit_FunctionDef(node)
         self.register_function_annotations(node)
         self.generic_visit(node)
 
-    def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef) -> None:
+    def visit_AsyncFunctionDef(self, node: AsyncFunctionDef) -> None:
         """Remove and map function argument- and return annotations."""
         super().visit_AsyncFunctionDef(node)
         self.register_function_annotations(node)
